
#ifndef MINAKO_H_INCLUDED
#define MINAKO_H_INCLUDED

/* ************************************************************* Präprozessor */

#include "symtab.h"
#include "stack.h"
#include "minako-syntax.tab.h"
#include <stdio.h>

/* Symbolklassen */
#define MINAKO_CLASS_VARIABLE  'var'
#define MINAKO_CLASS_FUNCTION  'func'

/* Datentypen */
#define MINAKO_TYPE_BOOL  'bool'
#define MINAKO_TYPE_INT   'int'
#define MINAKO_TYPE_FLOAT 'real'
#define MINAKO_TYPE_VOID  'void'

/* *************************************************************** Strukturen */

/**@brief Assoziierte Daten einer Minako Variablen.
 */
typedef struct
{
	long type,              /**< Datentyp */
	     line;              /**< Zeile der Deklaration */
} minakoVariable;

/**@brief Assoziierte Daten einer Minako Funktion.
 */
typedef struct
{
	long type,              /**< Rückgabetyp */
	     line;              /**< Zeile der Deklaration */
	symtabSymbol** param;   /**< statisches Parameterfeld */
	int argCount;           /**< Anzahl der Parameter */
} minakoFunction;

/* *************************************************** Importierte Funktionen */

extern YYSTYPE yylval;
extern int yylex();
extern FILE* yyin;
extern int yylineno;
extern void yyerror(const char*, ...);

/* ********************************************************** Hilfsfunktionen */

/**@brief Erstellt ein neues Symboltabellen-Symbol, das von der Klasse einer
 * Minako-Variablen ist.
 * @note Das Symbol wird nur erstellt, nicht in die Symboltabelle geschrieben.
 * 
 * @param type  nativer Datentyp
 * @param id    Name
 * 
 * @return Zeiger auf das neu erstellte Symbol
 */
extern symtabSymbol* newMinakoVariable(long type, const char* id);

/**@brief Fügt eine Variable der aktuellen Schicht in der Symboltabelle hinzu.
 * @note Das Symbol wird kopiert. Der Aufrufer ist für die Freigabe des
 * Speichers des Symbols verantwortlich.
 * 
 * @param table  Ziel-Symboltabelle
 * @param sym    Minako Variable
 */
extern void minakoVariableDeclare(symtab* table, const symtabSymbol* sym);

/**@brief Erstellt ein neues Symboltabellen-Symbol, das eine Minako Funktion
 * repräsentiert.
 * 
 * @param type  nativer Datentyp
 * @param id    Name
 * 
 * @return Zeiger auf das neu erstellte Symbol
 */
extern symtabSymbol* newMinakoFunction(long type, const char* id);

/**@brief Löscht die assoziierte Datenstruktur einer Minako Funktion.
 */
extern void deleteMinakoFunction(minakoFunction* func);

/**@brief Fügt die Funktion der aktuellen Schicht der Symboltabelle hinzu.
 * @note Das Symbol wird kopiert. Der Aufrufer ist für die Freigabe des
 * Speichers des Symbols verantwortlich.
 * 
 * @param table      Ziel-Symboltabelle
 * @param sym        Minako Funktion
 * @param parameter  enthält die Parameter der Funktion auf einem Stack als
 *                   Minako Variablen
 * @note Der Stack wird automatisch freigegeben.
 */
extern void minakoFunctionDeclare(symtab* table, const symtabSymbol* sym,
     	                            stack* parameter);

#endif
